import os
import sys
import h5py
import json
import numpy as np
import random
import argparse

class QueryPrepare:
    def __init__(self, args):
        self.input_data = args.input_data
        self.query_count = int(args.query_count)
        self.table_name = args.table_name
        self.data_format = args.data_format
        if not self.data_format:
            ind = self.input_data.rfind(".")
            self.data_format = self.input_data[ind+1:]
            if self.data_format not in ['hdf5', 'fvecs', 'bvecs', 'ivecs']:
               self.data_format = 'hdf5'
        self.output_data = 'query.data'
        self.normalize = args.normalize
        self.data_dir = "data/"
        if not os.path.exists(self.data_dir):
            os.system("mkdir %s" % self.data_dir)

    def hdf5_read(self):
        cols = ["/train"]
        data = h5py.File(self.input_data, 'r')
        entities = np.concatenate([data[i] for i in cols])
        return entities

    def bvecs_read(self):
        a = np.fromfile(self.input_data, dtype=np.int32, count=1)
        b = np.fromfile(self.input_data, dtype=np.uint8)
        d = a[0]
        return b.reshape(-1, d + 4)[:, 4:].copy()

    def ivecs_read(self):
        a = np.fromfile(self.input_data, dtype='int32')
        d = a[0]
        return a.reshape(-1, d + 1)[:, 1:].copy()

    def fvecs_read(self):
        return self.ivecs_read().view('float32')

    def write_query(self, entities):
        output_file = open(os.path.join(self.data_dir, self.output_data), "w")
        for cnt, row in enumerate(entities):
            j = {
                "tableName": self.table_name,
                "vector": row,
                "topK": 10
            }
            output_file.write(json.dumps(j) + "\n")
            print(cnt, end="\r")
        output_file.close()
        print(len(entities), len(entities[0]))

    def prepare(self):
        print("Reading %s data..." % self.data_format)
        data = []
        if self.data_format == "hdf5":
            data = self.hdf5_read()
        elif self.data_format == "bvecs":
            data = self.bvecs_read()
        elif self.data_format == "ivecs":
            data = self.ivecs_read()
        elif self.data_format == "fvecs":
            data = self.fvecs_read()
        if self.normalize:
            data = preprocessing.normalize(data, axis=1, norm='l2')
        print("Randomly Select Query...")
        queries = random.sample(data.tolist(), self.query_count)
        self.write_query(queries)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input_data", help="input data", action="store", required=True)
    parser.add_argument("-f", "--data_format", help="input data format", action="store", choices = ['hdf5', '', 'fvecs', 'bvecs', 'ivecs'], default=None)
    parser.add_argument("-c", "--query_count", help="query count", action="store", required=True)
    parser.add_argument("-t", "--table_name", help="table name", action="store", required=True)
    parser.add_argument("--normalize", help="normalize data", action="store_true", default=False)
    args = parser.parse_args()
    process = QueryPrepare(args)
    process.prepare()

if __name__ == '__main__':
    try:
        main()
        print("prepare query success!")
    except Exception as e:
        print(e)
        sys.exit(1)